use std::sync::Arc;

use bevy_ecs::system::Resource;
use vulkano::{
    command_buffer::allocator::StandardCommandBufferAllocator,
    descriptor_set::allocator::StandardDescriptorSetAllocator,
    device::{Device, Queue},
    instance::Instance,
    memory::allocator::StandardMemoryAllocator,
    swapchain::Surface,
};

use crate::core::{app::App, window::raw_handle::DisplayHandleWrapper};

use super::utils;

#[derive(Resource)]
pub struct VulkanContext {
    pub instance: Arc<Instance>,
    pub device: Arc<Device>,
    pub graphics_queue: Arc<Queue>,

    pub memory_allocator: Arc<StandardMemoryAllocator>,
    pub command_buffer_allocator: Arc<StandardCommandBufferAllocator>,
    pub descriptor_set_allocator: Arc<StandardDescriptorSetAllocator>,
}

impl From<&App> for VulkanContext {
    fn from(app: &App) -> Self {
        let library = utils::load_library();

        let world = app.world();

        let display_handle: &DisplayHandleWrapper =
            world.get_resource::<DisplayHandleWrapper>().unwrap();

        let enabled_extensions = Surface::required_extensions(&display_handle.0).unwrap();

        let instance = utils::create_instance(library.clone(), enabled_extensions);

        let (device, mut queues) = utils::pick_graphics_device(&instance, &display_handle.0);
        let graphics_queue = queues.next().unwrap();

        let memory_allocator = Arc::new(StandardMemoryAllocator::new_default(device.clone()));

        let command_buffer_allocator = Arc::new(StandardCommandBufferAllocator::new(
            device.clone(),
            Default::default(),
        ));

        let descriptor_set_allocator = Arc::new(StandardDescriptorSetAllocator::new(
            device.clone(),
            Default::default(),
        ));

        Self {
            instance: instance.clone(),
            device,
            graphics_queue,
            memory_allocator,
            command_buffer_allocator,
            descriptor_set_allocator,
        }
    }
}
